from appium_selector.CapGenerators.Caps import Caps, propertyFromString


class GridWeb(Caps):

    options = {}
    caps = {}

    def __init__(self, environment):
        self.env = environment

    def displayString(self):
        platform = propertyFromString('platform', self.env)
        browser = propertyFromString('browserName', self.env)
        osv = propertyFromString('version', self.env)
        return "%s -- %s -- %s" % (platform, browser, osv)

    def desiredCaps(self, mustard=True):
        self.options['provider'] = 'gridWeb'
        self.options['manufacturer'] = propertyFromString('platform', self.env)
        self.options['model'] = propertyFromString('browserName', self.env)
        self.options['osv'] = 'Local'
        self.options['mustard'] = mustard

        self.caps['platformName'] = propertyFromString('platform', self.env)
        self.caps['browserName'] = propertyFromString('browserName', self.env)
        self.caps['udid'] = self.displayString()
        self.caps['deviceName'] = self.displayString()
        self.caps['version'] = propertyFromString('version', self.env)
        return {'desiredCaps': self.caps, 'options': self.options}



from tkinter import *
import sys
import os
from tkinter.ttk import Notebook

from appium_selector.Helpers.Config import GetConfig
from appium_selector.Helpers.GridConnector import GridConnector
from appium_selector.Helpers.LocalConnector import LocalConnector
from appium_selector.Helpers.SauceConnector import SauceConnector


class DeviceSelector:

    BASE_DIR = os.path.abspath(os.path.dirname(__file__))
    devices = None

    local = LocalConnector()
    grid = GridConnector()
    sauce = SauceConnector()

    def __init__(self, parallel=False, platform='mobile'):
        self.parallel = parallel
        self.platform = platform


    def start(self):
        parallel = self.parallel
        platform = self.platform

        # Create Window
        self.root = Tk()
        win = self.root
        self.note = Notebook(self.root)

        # Set Window Properties
        if sys.platform.lower() == 'windows' or sys.platform.lower() == 'win32':
            win.iconbitmap(default=os.path.join(self.BASE_DIR, 'runner.ico'))
        win.wm_title("Test Runner")
        win.minsize(width=500, height=500)

        # Create Listbox
        scrollbar = Scrollbar(win)
        scrollbar.pack(side=RIGHT, fill=Y)

        #Add Mustard Checkbox and Instructions Label
        frame = Frame(win)


        self.mustard = GetConfig('SKIP_MUSTARD').lower() == 'false'
        c = Checkbutton(frame, text="Add Mustard?", command=self._toggleMustard)
        if self.mustard:
            c.select()
        c.pack(side=RIGHT)

        mobileFrame = Frame(win)
        if parallel:
            label = Label(frame, text="Please Select one or more devices to run")
            label.pack(side=LEFT)
            self.listboxMobile = Listbox(mobileFrame, selectmode=EXTENDED)
        else:
            label = Label(frame, text="Please Select one device to run")
            label.pack(side=LEFT)
            self.listboxMobile = Listbox(mobileFrame, selectmode=SINGLE)
        frame.pack(fill=X)


        self.listboxMobile.pack(fill=BOTH, expand=1)

        # Attach Scrollbar to Listbox
        self.listboxMobile.config(yscrollcommand=scrollbar.set)
        scrollbar.config(command=self.listboxMobile.yview)

        # Generate Listbox Data
        mobileNodes = self.grid.mobileNodes
        for node in mobileNodes:
            self.listboxMobile.insert(END, node.displayString())
        self.mobileData = mobileNodes

        self.listboxMobile.insert(END, 'SauceLabs')
        self.listboxMobile.insert(END, 'Local Device')

        desktopFrame = Frame(win)
        if parallel:
            self.listboxDesktop = Listbox(desktopFrame, selectmode=EXTENDED)
        else:
            self.listboxDesktop = Listbox(desktopFrame, selectmode=SINGLE)
        frame.pack(fill=X)


        self.listboxDesktop.pack(fill=BOTH, expand=1)

        # Attach Scrollbar to Listbox
        self.listboxDesktop.config(yscrollcommand=scrollbar.set)
        scrollbar.config(command=self.listboxDesktop.yview)

        # Generate Listbox Data
        webData = []
        webDisplay =[]

        localNodes = self.local.webNodes
        for node in localNodes:
            webData.append(node)
            self.listboxDesktop.insert(END, node.displayString())

        webNodes = self.grid.webNodes
        for node in webNodes:
            ds = node.displayString()
            if ds not in webDisplay:
                webDisplay.append(ds)
                webData.append(node)
                self.listboxDesktop.insert(END, ds)

        sauceNodes = self.sauce.webNodes
        for node in sauceNodes:
            webData.append(node)
            self.listboxDesktop.insert(END, node.displayString())
        self.webData = webData

        self.frame = Frame(win)
        self.frame.pack(fill=X)


        # Create Buttons
        Button(mobileFrame, text="Cancel", fg="red", command=self.frame.quit, width=50, height=5).pack(side=RIGHT, fill=Y)
        Button(mobileFrame, text="Run Test", fg="green", command=self._saveDevices, width=50).pack(side=LEFT, fill=Y)

        # Create Buttons
        Button(desktopFrame, text="Cancel", fg="red", command=self.frame.quit, width=50, height=5).pack(side=RIGHT, fill=Y)
        Button(desktopFrame, text="Run Test", fg="green", command=self._saveDevicesDesktop, width=50).pack(side=LEFT, fill=Y)

        if platform.lower() == 'mobile':
            self.note.add(mobileFrame, text='Mobile')
            self.note.add(desktopFrame, text='Web')
        else:
            self.note.add(desktopFrame, text='Web')
            self.note.add(mobileFrame, text='Mobile')
        self.note.pack(fill=BOTH, expand=1)

    def _toggleMustard(self):
        self.mustard = not self.mustard

    def getDevice(self):
        self.start()
        self.root.mainloop()
        self.root.destroy()
        output = []
        for device in self.devices:
            output.append(device.desiredCaps(mustard=self.mustard))
        return output

    def _saveDevicesDesktop(self):

        selected = self.listboxDesktop.curselection()

        output = []
        for selection in selected:
            output.append(self.webData[int(selection)])

        self.frame.quit()
        self.devices = output

    def _saveDevices(self):

        selected = self.listboxMobile.curselection()

        output = []
        for selection in selected:
            output.append(self.mobileData[int(selection)])

        self.frame.quit()
        self.devices = output



from appium_selector.CapGenerators.LocalWeb import LocalWeb
from appium_selector.Helpers.Config import GetConfig


class LocalConnector(object):

    localWeb = ['ie', 'chrome', 'firefox']
    webNodes = []

    def __init__(self):

        for browser in self.localWeb:
            self.webNodes.append(LocalWeb(browser))



from appium_selector.CapGenerators.Caps import Caps


class SauceMobile(Caps):

    sauceNode = []
    options = {}
    caps = {}

    def __init__(self, environment):
        self.env = environment

    def displayString(self):
        platform = self.env.find('platformName').text
        browser = self.env.find('browserName').text
        version = self.env.find('version').text
        return "Sauce -- %s -- %s -- %s" % (platform, browser, version)

    def desiredCaps(self, mustard=True):
        self.options['provider'] = 'sauceWeb'
        self.options['manufacturer'] = self.env.find('platformName').text
        self.options['model'] = self.env.find('browserName').text
        self.options['osv'] = self.env.find('version').text
        self.options['mustard'] = mustard

        self.caps['platformName'] = self.env.find('platformName').text
        self.caps['browserName'] = self.env.find('browserName').text
        self.caps['version'] = self.env.find('version').text
        return {'desiredCaps': self.caps, 'options': self.options}



from appium_selector.CapGenerators.Caps import Caps, propertyFromString
from appium_selector.Helpers.GetDeviceInfo import getDeviceInfo


class GridMobile(Caps):

    options = {}
    caps = {}

    def __init__(self, environment):
        self.env = environment
        self.udid = propertyFromString('deviceName', self.env)
        self.platform = propertyFromString('platform', self.env)
        self.device = getDeviceInfo(self.udid, self.platform)

    def displayString(self):
        platform = self.device['platform']
        udid = self.device['udid']
        name = self.device['name']
        return "%s -- %s -- %s" % (platform, udid, name)
    
    def desiredCaps(self, mustard=True):
        self.options['provider'] = 'gridMobile'
        self.options['manufacturer'] = self.device['manufacturer']
        self.options['model'] = self.device['model']
        self.options['osv'] = self.device['osv']
        self.options['mustard'] = mustard
        self.caps['platformName'] = propertyFromString('platform', self.env)
        self.caps['browserName'] = self.device['udid']
        self.caps['udid'] = self.device['udid']
        self.caps['deviceName'] = self.device['name']
        
        return {'desiredCaps': self.caps, 'options': self.options}


import os
import xml.etree.ElementTree as ET
from appium_selector.CapGenerators.SauceMobile import SauceMobile
from appium_selector.CapGenerators.SauceWeb import SauceWeb
from appium_selector.Helpers.Config import GetConfig


class SauceConnector(object):

    webNodes = []
    mobileNodes = []

    def __init__(self):

        #Open Device Config File
        projectFolder = GetConfig("ProjectFolder")
        devicePath = projectFolder + "/Sauce.xml"
        if not os.path.exists(devicePath):
            open(devicePath, 'w').close()

        #Find Device
        root = ET.parse(devicePath).getroot()
        webNodes = root.findall(".//node[platform='web']")
        for node in webNodes:
            self.webNodes.append(SauceWeb(node))

        mobileNodes = root.findall(".//node[platform='mobile']")
        for node in mobileNodes:
            pass
            self.mobileNodes.append(SauceMobile(node))


from setuptools import setup
setup(
  name = 'appium_selector',
  packages = ['appium_selector', 'appium_selector.Helpers', 'appium_selector.CapGenerators'],
  version = '1.0.19',
  include_package_data = True,
  description = 'Creates a Tcl/Tk window to read connected Appium devices from Selenium Grid and display them for selection',
  author = 'Matt Watson',
  author_email = 'Watson.Mattc@gmail.com',
  url = 'https://github.com/Mattwhooo/appium_selector.git',
  download_url = 'https://github.com/Mattwhooo/appium_selector/tarball/1.0.19',
  keywords = ['appium', 'selenium', 'testing'],
  classifiers=[],
  install_requires=['beautifulsoup4', 'requests'],
  package_data={
   'appium_selector.Helpers': ['*'],     # All files from folder A
   'appium_selector.CapGenerators': ['*'],     # All files from folder A
   }
)


from appium_selector.CapGenerators.Caps import Caps, propertyFromString


class LocalWeb(Caps):

    options = {}
    caps = {}

    def __init__(self, browser):
        self.browser = browser

    def displayString(self):
        return "Local %s" % self.browser

    def desiredCaps(self, mustard=False):
        self.options['provider'] = 'local-' + self.browser
        self.options['manufacturer'] = 'local'
        self.options['model'] = 'local'
        self.options['osv'] = 'Local'
        self.options['mustard'] = False

        self.caps['platformName'] = 'Local'
        self.caps['browserName'] = 'Local'
        self.caps['version'] = 'Local'
        self.caps['deviceName'] = 'Local'

        return {'desiredCaps': self.caps, 'options': self.options}



import os
import xml.etree.ElementTree as ET

from appium_selector.Helpers.Config import GetConfig


def getDeviceInfo(udid, platform):

    #Open Device Config File
    projectFolder = GetConfig("ProjectFolder")
    devicePath = projectFolder + "/Devices.xml"
    if not os.path.exists(devicePath):
        open(devicePath, 'w').close()

    #Find Device
    root = ET.parse(devicePath).getroot()
    deviceNode = root.findall(".//device[udid='" + udid + "']")

    try:
        device = {}
        device['udid'] = udid
        device['platform'] = platform
        device['name'] = deviceNode[0].find('deviceName').text
        device['manufacturer'] = deviceNode[0].find('manufacturer').text
        device['model'] = deviceNode[0].find('model').text
        device['osv'] = deviceNode[0].find('osv').text
        return device
    except:
        device = {}
        device['udid'] = udid
        device['platform'] = platform
        device['name'] = 'unknown'
        device['manufacturer'] = 'unknown'
        device['model'] = 'unknown'
        device['osv'] = 'unknown'
        return device


import re

import requests
from bs4 import BeautifulSoup

from appium_selector.CapGenerators.GridMobile import GridMobile
from appium_selector.CapGenerators.GridMobileWeb import GridMobileWeb
from appium_selector.CapGenerators.GridWeb import GridWeb
from appium_selector.Helpers.Config import GetConfig


class GridConnector(object):

    webNodes = []
    mobileNodes = []

    browsers = ['chrome', 'firefox', 'ie', 'safari', 'edge']
    mobilePlatforms = ['ANDROID', 'IOS']

    def __init__(self):
        for node in self._getNodes():
            self._parseNodes(node)

    def _getNodes(self):
        try:
            page = requests.get(GetConfig('GRID_URL') + '/grid/console')
            soup = BeautifulSoup(page.text, 'html.parser')
            resultsWeb = soup.select('img[title*=WebDriver]')
            resultsMobile = soup.select('a[title*=platform]')
            return resultsWeb + resultsMobile
        except:
            return []

    def _parseNodes(self, node):
        if self._propertyFromTitle('platform', node['title']) not in self.mobilePlatforms:
            self.webNodes.append(GridWeb(node['title']))
        elif self._propertyFromTitle('browserName', node['title']) in self.browsers:
            self.webNodes.append(GridMobileWeb(node['title']))
        else:
            self.mobileNodes.append(GridMobile(node['title']))

    def _propertyFromTitle(self, property, title):
        return re.search(property + '=.*?[}|,]', title).group().split('=')[1].replace('}','').replace(',','')

